unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ComCtrls, StdCtrls, ExtCtrls, ActnList, Grids, UAcronym, UPersistentForm,
  URegistry, ULastOpenedList, UListViewSort, UJobProgressView, UCoolTranslator,
  Registry, SpecializedList, LazUTF8, LazFileUtils;

type

  { TFormMain }

  TFormMain = class(TForm)
    AProcessImports: TAction;
    AShowImportFormats: TAction;
    AShowAbout: TAction;
    AShowImportSources: TAction;
    AShowAcronyms: TAction;
    AShowCategories: TAction;
    ASettings: TAction;
    AFileSaveAs: TAction;
    AFileClose: TAction;
    AFileSave: TAction;
    AFileNew: TAction;
    AFileOpen: TAction;
    AManualImport: TAction;
    AShow: TAction;
    AExit: TAction;
    ActionList1: TActionList;
    CheckBoxExactMath: TCheckBox;
    CoolTranslator1: TCoolTranslator;
    ImageList1: TImageList;
    JobProgressView1: TJobProgressView;
    LastOpenedList1: TLastOpenedList;
    ListViewAcronyms: TListView;
    ListViewFilter1: TListViewFilter;
    ListViewSort1: TListViewSort;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItemToolbar: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    PersistentForm1: TPersistentForm;
    PopupMenuTryIcon: TPopupMenu;
    SaveDialog1: TSaveDialog;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    TrayIcon1: TTrayIcon;
    procedure AExitExecute(Sender: TObject);
    procedure AFileCloseExecute(Sender: TObject);
    procedure AFileNewExecute(Sender: TObject);
    procedure AFileOpenExecute(Sender: TObject);
    procedure AFileSaveAsExecute(Sender: TObject);
    procedure AFileSaveExecute(Sender: TObject);
    procedure AManualImportExecute(Sender: TObject);
    procedure AProcessImportsExecute(Sender: TObject);
    procedure ASettingsExecute(Sender: TObject);
    procedure AShowAboutExecute(Sender: TObject);
    procedure AShowAcronymsExecute(Sender: TObject);
    procedure AShowCategoriesExecute(Sender: TObject);
    procedure AShowExecute(Sender: TObject);
    procedure AShowImportFormatsExecute(Sender: TObject);
    procedure AShowImportSourcesExecute(Sender: TObject);
    procedure CheckBoxExactMathChange(Sender: TObject);
    procedure CoolTranslator1Translate(Sender: TObject);
    procedure EditSearchChange(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LastOpenedList1Change(Sender: TObject);
    procedure ListViewAcronymsData(Sender: TObject; Item: TListItem);
    procedure ListViewAcronymsResize(Sender: TObject);
    procedure ListViewAcronymsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure ListViewFilter1Change(Sender: TObject);
    function ListViewSort1CompareItem(Item1, Item2: TObject): Integer;
    procedure ListViewSort1Filter(ListViewSort: TListViewSort);
    procedure MenuItemToolbarClick(Sender: TObject);
    procedure TrayIcon1Click(Sender: TObject);
  private
    Initialized: Boolean;
    FAlwaysOnTop: Boolean;
    RegistryContext: TRegistryContext;
    ProjectClosed: Boolean;
    ImportTotalItemCount: Integer;
    function FindFirstNonOption: string;
    procedure ProjectOpen(FileName: string);
    procedure Initialize;
    procedure ProcessImportsJob(Job: TJob);
    procedure SetAlwaysOnTop(AValue: Boolean);
    procedure FilterList(List: TListObject);
    procedure OpenRecentClick(Sender: TObject);
    procedure UpdateAcronymsList;
    procedure UpdateInterface;
    procedure LoadConfig;
    procedure SaveConfig;
  public
    AcronymDb: TAcronymDb;
    StartOnLogon: Boolean;
    StartMinimizedToTray: Boolean;
    function CompareStrings(Strings1, Strings2: TStrings): Boolean;
    property AlwaysOnTop: Boolean read FAlwaysOnTop write SetAlwaysOnTop;
  end;

var
  FormMain: TFormMain;

resourcestring
  SAddedCount = 'Imported %d acronyms. Added %d new.';
  SProcessImportSources = 'Process import sources';


implementation

{$R *.lfm}

uses
  UFormImport, UFormSettings, UFormCategories, UFormAcronyms,
  UFormImportSources, UFormAbout, UFormImportFormats;

resourcestring
  SModified = 'modified';
  SAppExit = 'Application exit';
  SAppExitQuery = 'Acronyms were modified. Do you want to save them to file before exit?';

const
  ProjectExt = '.adp';
  DefaultFileName = 'Acronyms' + ProjectExt;
  DefaultRegKey = '\Software\Chronosoft\Acronym Decoder';
  RegistryRunKey = '\Software\Microsoft\Windows\CurrentVersion\Run';
  ExampleFile = 'Example acronyms.adp';

{ TFormMain }

procedure TFormMain.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  AcronymDb := TAcronymDb.Create;
  Initialized := False;
  for I := 0 to ToolBar1.ButtonCount - 1 do
    ToolBar1.Buttons[I].Hint := ToolBar1.Buttons[I].Caption;
end;

procedure TFormMain.EditSearchChange(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.FormHide(Sender: TObject);
begin
  PersistentForm1.Save(Self);
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  PersistentForm1.Save(Self);
  SaveConfig;
end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  AFileClose.Execute;
  CanClose := ProjectClosed;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AFileCloseExecute(Sender: TObject);
var
  ModalResult: TModalResult;
  DoClose: Boolean;
begin
  DoClose := False;
  if Assigned(AcronymDb) then begin
    if AcronymDb.Modified then begin
       ModalResult := MessageDlg(SAppExit, SAppExitQuery,
       mtConfirmation, [mbYes, mbNo, mbCancel], 0);
      if ModalResult = mrYes then begin
        AFileSave.Execute;
        DoClose := True;
      end
      else if ModalResult = mrNo then begin
        DoClose := True;
      end else ProjectClosed := False;
    end else DoClose := True;
  end;
  if DoClose then begin
    FreeAndNil(AcronymDb);
    UpdateAcronymsList;
    UpdateInterface;
    ProjectClosed := True;
  end;
end;

procedure TFormMain.AFileNewExecute(Sender: TObject);
begin
  AFileClose.Execute;
  if ProjectClosed  then begin
    AcronymDb := TAcronymDb.Create;
    AcronymDb.FileName := DefaultFileName;
    AcronymDb.Acronyms.Clear;
    UpdateAcronymsList;
    UpdateInterface;
  end;
end;

procedure TFormMain.AFileOpenExecute(Sender: TObject);
begin
  OpenDialog1.DefaultExt := ProjectExt;
  if Assigned(AcronymDb) then
    OpenDialog1.FileName := AcronymDb.FileName;
  if OpenDialog1.Execute then begin
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AFileSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.DefaultExt := ProjectExt;
  SaveDialog1.FileName := AcronymDb.FileName;
  if SaveDialog1.Execute then begin
    AcronymDb.SaveToFile(SaveDialog1.FileName);
    LastOpenedList1.AddItem(SaveDialog1.FileName);
    UpdateInterface;
  end;
end;

procedure TFormMain.AFileSaveExecute(Sender: TObject);
begin
  if FileExists(AcronymDb.FileName) then begin
    AcronymDb.SaveToFile(AcronymDb.FileName);
    LastOpenedList1.AddItem(AcronymDb.FileName);
    UpdateInterface;
  end else AFileSaveAs.Execute;
end;

procedure TFormMain.AManualImportExecute(Sender: TObject);
begin
  FormImport.ShowModal;
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.AProcessImportsExecute(Sender: TObject);
begin
  ImportTotalItemCount := 0;
  AcronymDb.AddedCount := 0;
  JobProgressView1.AddJob(SProcessImportSources, ProcessImportsJob);
  JobProgressView1.Start;
  ShowMessage(Format(SAddedCount, [ImportTotalItemCount, AcronymDb.AddedCount]));
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.ProcessImportsJob(Job: TJob);
var
  I: Integer;
begin
  for I := 0 to AcronymDb.ImportSources.Count - 1 do
  with TImportSource(AcronymDb.ImportSources[I]) do
  if Enabled then begin
    Process;
    ImportTotalItemCount := ImportTotalItemCount + ItemCount;
    Job.Progress.Max := AcronymDb.ImportSources.Count;
    Job.Progress.Value := I;
    if Job.Terminate then Break;
  end;
end;

procedure TFormMain.ASettingsExecute(Sender: TObject);
begin
  FormSettings.Load;
  if FormSettings.ShowModal = mrOk then begin
    FormSettings.Save;
    SaveConfig;
  end;
end;

procedure TFormMain.AShowAboutExecute(Sender: TObject);
begin
  FormAbout.ShowModal;
end;

procedure TFormMain.AShowAcronymsExecute(Sender: TObject);
begin
  FormAcronyms.ShowModal;
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.AShowCategoriesExecute(Sender: TObject);
begin
  FormCategories.Categories := AcronymDb.Categories;
  FormCategories.ShowModal;
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.AShowExecute(Sender: TObject);
begin
  Show;
  BringToFront;
end;

procedure TFormMain.AShowImportFormatsExecute(Sender: TObject);
begin
  FormImportFormats.ImportFormats := AcronymDb.ImportFormats;
  FormImportFormats.ShowModal;
  UpdateInterface;
end;

procedure TFormMain.AShowImportSourcesExecute(Sender: TObject);
begin
  FormImportSources.ImportSources := AcronymDb.ImportSources;
  FormImportSources.ShowModal;
  UpdateAcronymsList;
  UpdateInterface;
end;

procedure TFormMain.CheckBoxExactMathChange(Sender: TObject);
begin
  UpdateAcronymsList;
end;

procedure TFormMain.CoolTranslator1Translate(Sender: TObject);
begin
  UAcronym.Translate;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(AcronymDb);
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  Initialize;

  PersistentForm1.Load(Self);
  UpdateInterface;
  ListViewFilter1.StringGrid.Col := 1;
  ListViewFilter1.StringGrid.Col := 0;
  ListViewFilter1.StringGrid.Row := 0;
  ListViewFilter1.StringGrid.SetFocus;
end;

procedure TFormMain.LastOpenedList1Change(Sender: TObject);
begin
  LastOpenedList1.LoadToMenuItem(MenuItemOpenRecent, OpenRecentClick);
end;

procedure TFormMain.ListViewAcronymsData(Sender: TObject; Item: TListItem);
begin
  if Item.Index < ListViewSort1.List.Count then
  with TAcronymMeaning(ListViewSort1.List[Item.Index]) do begin
    Item.Caption := Acronym.Name;
    Item.Data := TAcronymMeaning(ListViewSort1.List[Item.Index]);
    Item.SubItems.Add(Name);
    Item.SubItems.Add(Categories.GetString);
  end;
end;

procedure TFormMain.ListViewAcronymsResize(Sender: TObject);
begin
  ListViewFilter1.UpdateFromListView(ListViewAcronyms);
end;

procedure TFormMain.ListViewAcronymsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  UpdateInterface;
end;

procedure TFormMain.ListViewFilter1Change(Sender: TObject);
begin
  UpdateAcronymsList;
end;

function TFormMain.ListViewSort1CompareItem(Item1, Item2: TObject): Integer;
begin
  Result := 0;
  if Assigned(Item1) and Assigned(Item2) and (ListViewSort1.Order <> soNone) then begin
    with ListViewSort1 do
    case Column of
      0: Result := CompareString(TAcronymMeaning(Item1).Acronym.Name, TAcronymMeaning(Item2).Acronym.Name);
      1: Result := CompareString(TAcronymMeaning(Item1).Description, TAcronymMeaning(Item2).Description);
      2: Result := CompareString(TAcronymMeaning(Item1).Categories.GetString, TAcronymMeaning(Item2).Categories.GetString);
    end;
    if ListViewSort1.Order = soDown then Result := -Result;
  end else Result := 0;
end;

procedure TFormMain.ListViewSort1Filter(ListViewSort: TListViewSort);
begin
  AcronymDb.AssignToList(ListViewSort1.List);
  FilterList(ListViewSort1.List);
end;

procedure TFormMain.MenuItemToolbarClick(Sender: TObject);
begin
  MenuItemToolbar.Checked := not MenuItemToolbar.Checked;
  UpdateInterface;
end;

procedure TFormMain.TrayIcon1Click(Sender: TObject);
begin
  if not Visible then AShow.Execute
    else Hide;
end;

function TFormMain.FindFirstNonOption: string;
var
  S: string;
  I: Integer;
begin
  Result := '';
  for I := 1 to Application.ParamCount do begin
    S := Application.Params[I];
    if S[1] = Application.OptionChar then Continue;
    Result := S;
    Break;
  end;
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  AFileClose.Execute;
  if ProjectClosed then begin
    try
      AFileNew.Execute;
      AcronymDb.LoadFromFile(FileName);
      LastOpenedList1.AddItem(FileName);
    finally
      UpdateAcronymsList;
      UpdateInterface;
    end;
  end;
end;

procedure TFormMain.Initialize;
var
  FileNameOption: string;
begin
  if not Initialized then begin
    Initialized := True;
    LoadConfig;

    {$IFDEF 0}
    if Application.HasOption('h', 'help') then begin
      WriteLn('AcronymDecoder <project file>');
      WriteLn('  -t    --tray   Start minimized in system tray');
      WriteLn('  -h    --help  Show this help');
      Application.Terminate;
      Exit;
    end;
    {$ENDIF}

    if Application.HasOption('t', 'tray') then begin
      Visible := False;
    end;

    FileNameOption := FindFirstNonOption;
    if FileNameOption <> '' then begin
      // Open file specified as command line parameter
      AcronymDB.LoadFromFile(FileNameOption);
      LastOpenedList1.AddItem(OpenDialog1.FileName);
    end else
    if (LastOpenedList1.Items.Count > 0) and FileExists(LastOpenedList1.Items[0]) then begin
      // Open last opened file
      AcronymDB.LoadFromFile(LastOpenedList1.Items[0])
    end else begin
      // Open default database with examples if no item is in recent openned history
      FileNameOption := ExtractFileDir(Application.ExeName) + DirectorySeparator + ExampleFile;
      {$IFDEF Linux}
      // If installed in Linux system then use installation directory for po files
      if Application.ExeName = '/usr/bin/' + ExtractFileNameOnly(Application.ExeName) then
        FileNameOption := '/usr/share/' + ExtractFileNameOnly(Application.ExeName) + '/Examples/' + ExampleFile;
      {$ENDIF}
      ProjectOpen(FileNameOption);
    end;
    UpdateAcronymsList;
    ListViewFilter1.UpdateFromListView(ListViewAcronyms);
  end;
end;

function TFormMain.CompareStrings(Strings1, Strings2: TStrings): Boolean;
var
  I: Integer;
begin
  Result := Strings1.Count = Strings2.Count;
  if not Result then Exit;
  for I := 0 to Strings1.Count - 1 do
    if (Strings1[I] <> Strings2[I]) or (Strings1.Objects[I] <> Strings2.Objects[I]) then begin
      Result := False;
      Exit;
    end;
end;

procedure TFormMain.SetAlwaysOnTop(AValue: Boolean);
begin
  if FAlwaysOnTop = AValue then Exit;
  FAlwaysOnTop := AValue;
  if FAlwaysOnTop then FormStyle := fsSystemStayOnTop
    else FormStyle := fsNormal;
end;

procedure TFormMain.FilterList(List: TListObject);
var
  I: Integer;
  FoundCount: Integer;
  EnteredCount: Integer;
begin
  EnteredCount := ListViewFilter1.TextEnteredCount;
  for I := List.Count - 1 downto 0 do begin
    if List.Items[I] is TAcronymMeaning then begin
      with TAcronymMeaning(List.Items[I]) do begin
         with ListViewFilter1 do
         if Visible and (EnteredCount > 0) then begin
           FoundCount := 0;
           if CheckBoxExactMath.Checked then begin
             if TextEnteredColumn(0) and (StringGrid.Cells[0, 0] =
               TAcronymMeaning(List.Items[I]).Acronym.Name) then Inc(FoundCount);
             if TextEnteredColumn(1) and (StringGrid.Cells[1, 0] =
               TAcronymMeaning(List.Items[I]).Name) then Inc(FoundCount);
             if TextEnteredColumn(2) and (StringGrid.Cells[2, 0] =
               TAcronymMeaning(List.Items[I]).Categories.GetString) then Inc(FoundCount);
           end else begin
             if Pos(UTF8LowerCase(StringGrid.Cells[0, 0]),
               UTF8LowerCase(TAcronymMeaning(List.Items[I]).Acronym.Name)) > 0 then Inc(FoundCount);
             if Pos(UTF8LowerCase(StringGrid.Cells[1, 0]),
               UTF8LowerCase(TAcronymMeaning(List.Items[I]).Name)) > 0 then Inc(FoundCount);
             if Pos(UTF8LowerCase(StringGrid.Cells[2, 0]),
               UTF8LowerCase(TAcronymMeaning(List.Items[I]).Categories.GetString)) > 0 then Inc(FoundCount);
           end;
           if FoundCount <> EnteredCount then List.Delete(I);
         end else List.Delete(I);
      end;
    end else
    if TAcronymMeaning(List.Items[I]) is TAcronymMeaning then begin
      List.Delete(I);
    end;
  end;
end;

procedure TFormMain.OpenRecentClick(Sender: TObject);
begin
  AFileClose.Execute;
  if ProjectClosed then begin
    AFileNew.Execute;
    AcronymDb.LoadFromFile(TMenuItem(Sender).Caption);
    LastOpenedList1.AddItem(TMenuItem(Sender).Caption);
    UpdateAcronymsList;
    UpdateInterface;
  end;
end;

procedure TFormMain.UpdateAcronymsList;
begin
  if Assigned(AcronymDb) then begin
    ListViewSort1.Refresh;
  end else ListViewSort1.List.Clear;
end;

procedure TFormMain.UpdateInterface;
var
  Title: string;
begin
  ListViewAcronyms.Enabled := Assigned(AcronymDb);
  AFileClose.Enabled := Assigned(AcronymDb);
  AFileSave.Enabled := Assigned(AcronymDb) and AcronymDb.Modified;
  AFileSaveAs.Enabled := Assigned(AcronymDb);
  AManualImport.Enabled := Assigned(AcronymDb);
  AProcessImports.Enabled := Assigned(AcronymDb);
  AShowCategories.Enabled := Assigned(AcronymDb);
  AShowAcronyms.Enabled := Assigned(AcronymDb);
  AShowCategories.Enabled := Assigned(AcronymDb);
  AShowImportSources.Enabled := Assigned(AcronymDb);
  AShowImportFormats.Enabled := Assigned(AcronymDb);
  ToolBar1.Visible := MenuItemToolbar.Checked;

  Title := Application.Title;
  if Assigned(AcronymDb) and (ExtractFileName(AcronymDb.FileName) <> '') then
    if Assigned(AcronymDb) then Title := Title + ' - ' + ExtractFileName(AcronymDb.FileName);
  if Assigned(AcronymDb) and AcronymDb.Modified then Title := Title + ' (' + SModified + ')';
  Caption := Title;
end;

procedure TFormMain.LoadConfig;
begin
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey);
  PersistentForm1.RegistryContext := RegistryContext;
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey + '\RecentFiles');
  LastOpenedList1.LoadFromRegistry(RegistryContext);

  {$IFDEF Linux}
  // If installed in Linux system then use installation directory for po files
  if Application.ExeName = '/usr/bin/' + ExtractFileNameOnly(Application.ExeName) then
    CoolTranslator1.POFilesFolder := '/usr/share/' + ExtractFileNameOnly(Application.ExeName) + '/languages';
  {$ENDIF}

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(DefaultRegKey, True);
    if ValueExists('LanguageCode') then
      CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode(ReadStringWithDefault('LanguageCode', ''))
      else CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode('');
    AlwaysOnTop := ReadBoolWithDefault('AlwaysOnTop', False);
    StartMinimizedToTray := ReadBoolWithDefault('StartMinimizedToTray', False);
    CheckBoxExactMath.Checked := ReadBoolWithDefault('ExactMatch', False);
    MenuItemToolbar.Checked := ReadBoolWithDefault('ToolBarVisible', True);
  finally
    Free;
  end;

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegistryRunKey, True);
    StartOnLogon := ValueExists('Acronym Decoder');
  finally
    Free;
  end;
end;

procedure TFormMain.SaveConfig;
begin
  RegistryContext := RegContext(HKEY_CURRENT_USER, DefaultRegKey + '\RecentFiles');
  LastOpenedList1.SaveToRegistry(RegistryContext);

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(DefaultRegKey, True);
    if Assigned(CoolTranslator1.Language) and (CoolTranslator1.Language.Code <> '') then
      WriteString('LanguageCode', CoolTranslator1.Language.Code)
      else DeleteValue('LanguageCode');
    WriteBool('AlwaysOnTop', AlwaysOnTop);
    WriteBool('StartMinimizedToTray', StartMinimizedToTray);
    WriteBool('ExactMatch', CheckBoxExactMath.Checked);
    WriteBool('ToolBarVisible', MenuItemToolbar.Checked);
  finally
    Free;
  end;

  with TRegistryEx.Create do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey(RegistryRunKey, True);
    if StartOnLogon then begin
      if StartMinimizedToTray then WriteString('Acronym Decoder', Application.ExeName + ' /t')
        else WriteString('Acronym Decoder', Application.ExeName)
    end else DeleteValue('Acronym Decoder');
  finally
    Free;
  end;
end;

end.

